Imports System.Data.SqlClient

'
' Run some queries on the SOSOS database and send "alerts" to a list of
' administrators based upon the results.  This program is designed to run as
' a scheduled task and therefore has no user interface.
'
' Note: Requires an Alerts table that looks like this:
'
' CREATE TABLE [dbo].[Alerts](
'	[WatchedTable] [nvarchar](150) NOT NULL,
'	[ReportView] [nvarchar](150) NULL,
'	[WatchedCount] [int] NULL,
'	[Emails] [nvarchar](255) NULL,
'	[PCNames] [nvarchar](255) NULL,
'	[EventLog] [bit] NOT NULL,
' PRIMARY KEY NONCLUSTERED 
' (
'	[WatchedTable] ASC
' ) ON [PRIMARY]
' ) ON [PRIMARY]

'
Module Main

    Sub Main()
        Dim con As New SqlConnection
        Dim cmd As New SqlCommand
        Dim cmd_temp As New SqlCommand
        Dim cb As SqlCommandBuilder
        Dim da As New SqlDataAdapter
        Dim dt As New DataTable
        Dim dt_temp As DataTable
        Dim dr, dr_temp As DataRow
        Dim ResetOnly As Boolean = False
        Dim UpdateRequired As Boolean = False
        Dim Emails(), PCNames(), WatchedTable, ReportView As String
        Dim WatchedCount, Current_Count, i As Integer
        Dim el As New EventLog
        Dim alert As New Alerts
        Dim buf As String

        Console.WriteLine("AlertSOSOS")
        Console.WriteLine("Start: " & Now)

        ' Check for command line options for "Reset Counts Only"
        If My.Application.CommandLineArgs.Count = 1 AndAlso My.Application.CommandLineArgs(0).ToLower = "/r" Then
            ResetOnly = True
        End If

        ' Open the database now so we won't get any surprises later
        Try
            con.ConnectionString = My.Settings.ConnectString
            con.Open()
        Catch ex As Exception
            el.Source = "AlertSOSOS"
            el.WriteEntry("Can't open the database" & vbCr & ex.Message, EventLogEntryType.Error)
            Beep()
            Console.WriteLine("Can't open the database" & vbCrLf & ex.Message)
            System.Threading.Thread.Sleep(5000)
            Environment.ExitCode = 4313 'ERROR_DATABASE_FAILURE
            Exit Sub
        End Try

        ' fill the data table
        Try
            cmd.Connection = con
            cmd.CommandText = "Select * from Alerts"
            da.SelectCommand = cmd
            da.Fill(dt)

            ' Loop throught the list of tables/views that we watch for, and
            ' check to see if the record count is greater than the last interation.
            cmd_temp.Connection = con
            For Each dr In dt.Rows
                WatchedTable = dr("WatchedTable").ToString
                WatchedCount = CInt(dr("WatchedCount"))
                ReportView = dr("ReportView").ToString

                ' get the record count from the table/view
                cmd_temp.CommandText = "Select count(*) from [" & WatchedTable & "]"
                cmd_temp.Connection = con
                Current_Count = CInt(cmd_temp.ExecuteScalar)
                Console.WriteLine("Table: " & WatchedTable & ", Current: " & Current_Count & ", Watched: " & WatchedCount)

                ' get the changes from last week (via a View)
                dt_temp = New DataTable
                cmd_temp.CommandText = "Select * from [" & ReportView & "]"
                da.SelectCommand = cmd_temp
                da.Fill(dt_temp)

                ' do the deed?
                If WatchedCount < Current_Count And ResetOnly = False Then
                    ' OK, let's build a CSV version of the table
                    buf = ""
                    For i = 0 To dt_temp.Columns.Count - 1
                        buf &= Chr(34) & dt_temp.Columns(i).ColumnName & Chr(34)
                        If i < dt_temp.Columns.Count - 1 Then
                            buf &= ","
                        Else
                            buf &= vbCr
                        End If
                    Next
                    For Each dr_temp In dt_temp.Rows
                        For i = 0 To dt_temp.Columns.Count - 1
                            If Not IsDBNull(dr_temp(i)) Then
                                If dt_temp.Columns(i).DataType Is System.Type.GetType("System.String") Then
                                    buf &= Chr(34) & dr_temp(i).ToString & Chr(34)
                                Else
                                    buf &= dr_temp(i).ToString
                                End If
                            End If
                            If i < dt_temp.Columns.Count - 1 Then
                                buf &= ","
                            Else
                                buf &= vbCr
                            End If
                        Next
                    Next

                    Try
                        ' The Emails and PCNames fields are a comma-separated list
                        ' of People/Computers to recieve the alerts.
                        If Not IsDBNull(dr("Emails")) Then
                            Emails = Split(dr("Emails").ToString, ",")
                            alert.SendEmail(Emails, WatchedTable, buf)
                        End If
                        If Not IsDBNull(dr("PCNames")) Then
                            PCNames = Split(dr("PCNames").ToString, ",")
                            alert.SendMessage(PCNames, WatchedTable)
                        End If
                        If CBool(dr("EventLog")) Then
                            alert.SendEventLog(WatchedTable)
                        End If
                    Catch ex As Exception
                        el.Source = "AlertSOSOS"
                        el.WriteEntry("Can't perform one of the alert actions" & vbCr & ex.Message, EventLogEntryType.Warning)
                        Beep()
                        Console.WriteLine("Can't perform one of the alert actions" & vbCrLf & ex.Message)
                    End Try

                End If

                ' reset the values for next time?
                If WatchedCount <> Current_Count Then
                    dr("WatchedCount") = Current_Count
                    UpdateRequired = True
                End If
            Next

            ' update the database
            If UpdateRequired Then
                Try
                    da.SelectCommand = cmd
                    cb = New SqlCommandBuilder(da)
                    da.Update(dt)
                Catch ex As Exception
                    el.Source = "AlertSOSOS"
                    el.WriteEntry("Can't update the Alerts table" & vbCr & ex.Message, EventLogEntryType.Error)
                    Beep()
                    Console.WriteLine("Can't update the Alerts table" & vbCrLf & ex.Message)
                    System.Threading.Thread.Sleep(5000)
                    Environment.ExitCode = 4313 'ERROR_DATABASE_FAILURE
                    Exit Sub
                End Try
            End If

            con.Close()
            Console.WriteLine("Finished: " & Now)
            System.Threading.Thread.Sleep(1000)
        Catch ex As Exception
            el.Source = "AlertSOSOS"
            el.WriteEntry("Internal Error" & vbCr & ex.Message & vbCr & ex.StackTrace, EventLogEntryType.Error)
            Beep()
            Console.WriteLine("Internal Error" & vbCrLf & ex.Message)
            System.Threading.Thread.Sleep(5000)
            Environment.ExitCode = 1287 ' ERROR_UNIDENTIFIED_ERROR
        End Try
    End Sub

End Module
